library bucketeer;

import 'dart:io';

import 'package:flutter/services.dart';

import 'result.dart';

/// Bucketeer Flutter SDK
class Bucketeer {
  const Bucketeer._();

  static late final Bucketeer instance = Bucketeer._();

  static const MethodChannel _channel =
      MethodChannel('jp.bucketeer.plugin/flutter');

  Future<Result<void>> initialize({
    required String apiKey,
    required String endpoint,
    required String featureTag,
    bool debugging = false,
    int logSendingIntervalMillis = 60000,
    int logSendingMaxBatchQueueCount = 50,
    int pollingEvaluationIntervalMillis = 600000,
  }) async {
    return resultGuard(
      'initialize',
      await invokeMethod('initialize', argument: {
        'apiKey': apiKey,
        'endpoint': endpoint,
        'featureTag': featureTag,
        'debugging': debugging,
        'logSendingIntervalMillis': logSendingIntervalMillis,
        'logSendingMaxBatchQueueCount': logSendingMaxBatchQueueCount,
        'pollingEvaluationIntervalMillis': pollingEvaluationIntervalMillis,
      }),
    );
  }

  Future<Result<void>> setUser(
    String userId, {
    Map<String, String> userMap = const {},
  }) async {
    return resultGuard(
      'setUser',
      await invokeMethod('setUser', argument: {
        'userId': userId,
        ...userMap,
      }),
    );
  }

  Future<Result<String>> getStringVariation(
    String featureId, {
    String defaultValue = '',
  }) async {
    return resultGuard<String>(
      'getStringVariation',
      await invokeMethod('getStringVariation', argument: {
        'featureId': featureId,
        'defaultValue': defaultValue,
      }),
    );
  }

  Future<Result<int>> getIntVariation(
    String featureId, {
    int defaultValue = 0,
  }) async {
    return resultGuard<int>(
      'getIntVariation',
      await invokeMethod('getIntVariation', argument: {
        'featureId': featureId,
        'defaultValue': defaultValue,
      }),
    );
  }

  Future<Result<double>> getDoubleVariation(
    String featureId, {
    double defaultValue = 0.0,
  }) async {
    return resultGuard<double>(
      'getDoubleVariation',
      await invokeMethod('getDoubleVariation', argument: {
        'featureId': featureId,
        'defaultValue': defaultValue,
      }),
    );
  }

  Future<Result<bool>> getBoolVariation(
    String featureId, {
    bool defaultValue = false,
  }) async {
    return resultGuard<bool>(
        'getBoolVariation',
        await invokeMethod('getBoolVariation', argument: {
          'featureId': featureId,
          'defaultValue': defaultValue,
        }));
  }

  Future<Result<void>> track(
    String goalId, {
    double value = 0.0,
  }) async {
    return resultGuard(
      'track',
      await invokeMethod('track', argument: {
        'goalId': goalId,
        'value': value,
      }),
    );
  }

  Future<Result<void>> start() async {
    // Only run on Android
    if (Platform.isIOS) return Future.value(Result.success());
    return resultGuard('start', await invokeMethod('start'));
  }

  Future<Result<void>> stop() async {
    // Only run on Android
    if (Platform.isIOS) return Future.value(Result.success());
    return resultGuard('stop', await invokeMethod('stop'));
  }

  Result<T> resultGuard<T>(
    String method,
    Map<String, dynamic> result,
  ) {
    if (result['status']) {
      if (result['response'] != null) {
        return Result<T>.success(data: result['response']);
      } else {
        return Result.success();
      }
    } else {
      return Result.failure(result['errorMessage']);
    }
  }

  Future<Map<String, dynamic>> invokeMethod(
    String method, {
    Map<String, dynamic> argument = const {},
  }) async {
    return Map<String, dynamic>.from(
      await _channel.invokeMethod(method, argument),
    );
  }
}
